unit Main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Controls, Forms,
  Dialogs, Contnrs, Buttons, iDragDrop, Menus, ExtCtrls, StdCtrls;

type
  TMainForm = class(TForm)
    OpenFile: TOpenDialog;
    PopupMenu: TPopupMenu;
    PMenuShowInfo: TMenuItem;
    PMenuCheckOpen: TMenuItem;
    PMenuSetPassword: TMenuItem;
    PMenuDelFromList: TMenuItem;
    PnlButtons: TPanel;
    BtnAddFile: TBitBtn;
    ChkAddFile: TCheckBox;
    EdtExts: TEdit;
    BtnCheck: TBitBtn;
    BtnLetGo: TBitBtn;
    Panel1: TPanel;
    LstFiles: TListBox;
    Splitter1: TSplitter;
    MemResult: TMemo;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure BtnAddFileClick(Sender: TObject);
    procedure LstFilesDrawItem(Control: TWinControl; Index: Integer;
      Rect: TRect; State: TOwnerDrawState);
    procedure BtnCheckClick(Sender: TObject);
    procedure PMenuShowInfoClick(Sender: TObject);
    procedure LstFilesMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure BtnLetGoClick(Sender: TObject);
  private
    Dropper: TShellDrop;
    WorkSL: TStringList;
    procedure AddFiles(Files: TStrings);
    procedure DroppedAction(AFiles, AFolds: TStringList;
            AControl: TControl; APos: TPoint);
    procedure DoMessage(const Msg: string; const Level: Byte = 0);
    procedure DoAnalyze();
    procedure DoAnalyzeFile(const OutPath, FileName, Password: string);
  public

  end;

var
  MainForm: TMainForm;

implementation

uses
    StrUtils, ShellApi, iado, iadodata, iadoconst;

{$R *.dfm}


//  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  _CAO
function SimpleMessage(const Msg: string; const DlgType: TMsgDlgType; const Buttons: TMsgDlgButtons): integer;
var
    Dialog: TForm;
    Image : TImage;
begin
    Application.NormalizeTopMosts;

    Dialog := CreateMessageDialog(Trim(Msg), DlgType, Buttons);
    try
        Dialog.Caption := Application.Title + ' - ' + Dialog.Caption; 
        Image := TImage(Dialog.FindComponent('Image'));
        if (Image <> nil) then Image.Picture.Icon.Assign(Application.Icon);
        Result := Dialog.ShowModal;
    finally
        Dialog.Free;
    end;

    Application.RestoreTopMosts;
end;

//  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  
function SimpleQuery(const Prompt: string; var Value: string): Boolean;
begin
    Result := InputQuery(Application.Title, Prompt, Value);
end;

//  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  ShellExecute
procedure SimpleExecute(const Path: string);
begin
    ShellExecute(Application.Handle, nil, PChar(Path), nil, nil, SW_SHOW);
end;

//  ---------------------------------------------------------------------------
//  RXgN^
procedure TMainForm.FormCreate(Sender: TObject);
begin
    Self.Caption := Application.Title + ' - ' + Self.Caption; 
    WorkSL := TStringList.Create;
    Dropper := TShellDrop.Create(LstFiles, DroppedAction);
    Dropper.Start();
end;

//  ---------------------------------------------------------------------------
//  fXgN^
procedure TMainForm.FormDestroy(Sender: TObject);
begin
    Dropper.Free;
    WorkSL.Free;
end;

//  ---------------------------------------------------------------------------
//  t@Cǉ()
procedure TMainForm.DoMessage(const Msg: string; const Level: Byte = 0);
const
    S_LEVEL = '  ';
var
    T: string;
begin
    T := FormatDateTime('hh:nn:ss', Now());
    MemResult.Lines.Add(Format('%s %s%s', [T, DupeString(S_LEVEL, Level), Msg]));
end;

//  ---------------------------------------------------------------------------
//  t@Cǉ()
procedure TMainForm.AddFiles(Files: TStrings);
var
    MdbInfo: TMdbFile;
    Path, Path2: string;
    OldIndex, i, index: integer;
    SL: TStringList;
    ThisExt, AllowExt: string;
begin
    //  ݈ʒu
    OldIndex := LstFiles.ItemIndex;

    //  t@Ct@Ĉݗ
    SL := TStringList.Create;
    SL.Sorted := true;
    for i := 0 to MdbFiles.Count - 1 do
    begin
        MdbInfo := TMdbFile(MdbFiles[i]);
        SL.Add(MdbInfo.DispName);
    end;

    //  ɑ݂Ăꍇ͖(t@C͕s)
    for i := 0 to Files.Count - 1 do
    begin
        Path  := Files[i];
        Path2 := ExtractFileName(Path);
        if not SL.Find(Path2, index) then
        begin
            //  gq𒲂ׂ
            ThisExt  := UpperCase(ExtractFileExt(Path));
            AllowExt := UpperCase(EdtExts.Text);
            if (AnsiPos(ThisExt, AllowExt) > 0) then
            begin
                MdbFiles.Add(TMdbFile.Create(Path));
                SL.Add(Path2);
            end;
        end;
    end;

    SL.Free;

    //  sݒ
    LstFiles.Count := MdbFiles.Count;

    //  I
    if (OldIndex = -1) and (LstFiles.Count > 0) then OldIndex := 0;
    LstFiles.ItemIndex := OldIndex;
end;

//  ---------------------------------------------------------------------------
//  t@Cǉ(_CAOoR)
procedure TMainForm.BtnAddFileClick(Sender: TObject);
begin
    if OpenFile.Execute then
    begin
        if not ChkAddFile.Checked then MdbFiles.Clear;
        AddFiles(OpenFile.Files);
    end;
end;

//  ---------------------------------------------------------------------------
//  t@Cǉ(hbOAhhbvoR)
procedure   TMainForm.DroppedAction(AFiles, AFolds: TStringList;
            AControl: TControl; APos: TPoint);
begin
    if not ChkAddFile.Checked then MdbFiles.Clear;
    AddFiles(AFiles);
end;

//  ---------------------------------------------------------------------------
//  \̓t@Ĉ
procedure TMainForm.LstFilesDrawItem(Control: TWinControl; Index: Integer;
  Rect: TRect; State: TOwnerDrawState);
var
    LB: TListBox;
    MdbInfo: TMdbFile;
    OffsetX, OffsetY: integer;
    Item: string;
begin
    //  LXg
    LB := TListBox(Control);
    //  t@CNXo
    MdbInfo := TMdbFile(MdbFiles[Index]);
    //  \o
    Item :=  Format('%s t@C[%s] pX[h[%s]', [S_STATUS1[MdbInfo.Status], MdbInfo.DispName, MdbInfo.Password]);
    //  \ʒu
    OffsetX := 5;
    Inc(OffsetX, Rect.Left);
    OffsetY := ((Rect.Bottom - Rect.Top) - LB.Canvas.TextHeight(Item)) div 2;
    Inc(OffsetY, Rect.Top);
    //  `
    LB.Canvas.TextRect(Rect, OffsetX, OffsetY, Item);
end;

//  ---------------------------------------------------------------------------
//  ڑ`FbN(`FbN̂̂݉͑ΏۂƂȂ)
procedure TMainForm.BtnCheckClick(Sender: TObject);
var
    i, Count: integer;
    MdbInfo: TMdbFile;
    AdoJet: TAdoxJetBase;
begin
    Screen.Cursor := crHourGlass;
    PnlButtons.Enabled := false;
    AdoJet := TAdoxJetBase.Create;
    MemResult.Clear;
    try
        DoMessage('ڑ`FbNJn');
        Count := MdbFiles.Count;
        for i := 0 to Count - 1 do
        begin
            //  t@CNXo
            MdbInfo := TMdbFile(MdbFiles[i]);
            DoMessage(Format('[%2d/%2d] `FbNJn %s', [i + 1, Count, MdbInfo.DispName]));
            //  `FbN̏ꍇ̂
            if (MdbInfo.Status = fsUnknown) then
            begin
                try
                    //  ڑĂ݂
                    AdoJet.OpenConnection(MdbInfo.FileName, MdbInfo.Password);
                    //  ؒf
                    AdoJet.CloseConnection();
                    //  Ȃ
                    MdbInfo.Status := fsNoProblem;
                except
                    //  蔭
                    on E:Exception do
                    begin
                        MdbInfo.Msg := E.Message;
                        if (AnsiPos('pX[h', MdbInfo.Msg) > 0) then
                            MdbInfo.Status := fsNeedPassword
                        else
                            MdbInfo.Status := fsError;
                    end;
                end;
            end;
            DoMessage(Format('%s %s', [S_STATUS1[MdbInfo.Status], MdbInfo.Msg]));
            LstFiles.Repaint();
        end;
        DoMessage('ڑ`FbNI');
    finally
        AdoJet.Free;
        PnlButtons.Enabled := true;
        Screen.Cursor := crDefault;
    end;
end;

//  ---------------------------------------------------------------------------
//  |bvAbvj[\
procedure TMainForm.LstFilesMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
    index: integer;
    Pt: TPoint;
begin
    //  NbNʒuɃACe邩
    index := TListBox(Sender).ItemAtPos(Point(X, Y), true);

    //  ENbN̏ꍇ
    if (index > -1) and (Button = mbRight) then
    begin
        //  I
        TListBox(Sender).ItemIndex := index;
        //  |bvAbv
        Pt := TListBox(Sender).ClientToScreen(Point(X, Y));
        PopupMenu.Popup(Pt.X, Pt.Y);
    end;
end;

//  ---------------------------------------------------------------------------
//  낢ƃACeƂ̂
procedure TMainForm.PMenuShowInfoClick(Sender: TObject);
var
    index: integer;
    MdbInfo: TMdbFile;
    AdoJet: TAdoxJetBase;
    Temp: string;
begin
    //  IĂACe𒲂ׂ
    index := LstFiles.ItemIndex;
    if (index = -1) then exit;
    //  IuWFNgoĂ
    MdbInfo := TMdbFile(MdbFiles[index]);

    //  
    if (Sender = PMenuShowInfo) then
    begin
        WorkSL.Clear;
        WorkSL.Add(Format('t@C'#9'%s', [MdbInfo.DispName]));
        WorkSL.Add(Format('tpX@'#9'%s', [MdbInfo.FileName]));
        WorkSL.Add(Format('Xe[^X'#9'%s', [S_STATUS2[MdbInfo.Status]]));
        WorkSL.Add(Format('bZ[W'#9'%s', [MdbInfo.Msg]));
        WorkSL.Add(Format('pX[h'#9'%s', [MdbInfo.Password]));
        SimpleMessage(WorkSL.Text, mtInformation, [mbOK]);
    end
    else
    //  ڑ`FbN
    if (Sender = PMenuCheckOpen) then
    begin
        AdoJet := TAdoxJetBase.Create;
        try
            //  `FbN̏ꍇ̂
            if (MdbInfo.Status = fsUnknown) then
            begin
                try
                    //  ڑĂ݂
                    AdoJet.OpenConnection(MdbInfo.FileName, MdbInfo.Password);
                    //  ؒf
                    AdoJet.CloseConnection();
                    //  Ȃ
                    MdbInfo.Status := fsNoProblem;
                except
                    //  蔭
                    on E:Exception do
                    begin
                        MdbInfo.Msg := E.Message;
                        if (AnsiPos('pX[h', MdbInfo.Msg) > 0) then
                            MdbInfo.Status := fsNeedPassword
                        else
                            MdbInfo.Status := fsError;
                    end;
                end;
            end;
        finally
            AdoJet.Free;
        end;
    end
    else
    //  pX[hݒ
    if (Sender = PMenuSetPassword) then
    begin
        Temp := MdbInfo.Password;
        if (SimpleQuery('pX[hݒł܂B', Temp)) then
        begin
            MdbInfo.Password := Temp;
        end;
    end
    else
    //  폜
    if (Sender = PMenuDelFromList) then
    begin
        //  폜
        MdbFiles.Delete(index);
        //  sݒ
        LstFiles.Count := MdbFiles.Count;
        //  Iʒu
        if (LstFiles.Count > 0) then
        begin
            if (LstFiles.Count <= index) then Dec(index);
            LstFiles.ItemIndex := index;
        end;
    end;
    //  ĕ`
    LstFiles.Repaint();
end;

//  ---------------------------------------------------------------------------
//  ͊Jn
procedure TMainForm.BtnLetGoClick(Sender: TObject);
var
    Msg: string;
begin
    //  mF
    Msg := 'ڑ`FbNς݂̃t@C([' + S_STATUS1[fsNoProblem] + '] ̃t@C)͂܂B'
         + #13#10'Ԃ\\łB͂n߂܂H';
    if (SimpleMessage(Msg, mtConfirmation, [mbYes, mbNo]) = mrNo) then exit;
    DoAnalyze();
end;

//  ---------------------------------------------------------------------------
//  
procedure TMainForm.DoAnalyze();
var
    Tick: Cardinal;
    i, Count: integer;
    MdbInfo: TMdbFile;
    OutPath: string;
begin
    //  o̓tH_pX
//    OutPath := ExtractFilePath(ParamStr(0)) + FormatDateTime('"mdberput"yyyymmddhhnn', Now()) + '\';
    OutPath := ExtractFilePath(ParamStr(0)) + FormatDateTime('"mdberput"yyyymmdd', Now()) + '\';

    Screen.Cursor := crHourGlass;
    PnlButtons.Enabled := false;
    MemResult.Clear;
    try
        Tick := GetTickCount();
        DoMessage('͂Jn܂B');
        DoMessage(StringOfChar('-', 50));
        Count := MdbFiles.Count;

        for i := 0 to Count - 1 do
        begin
            //  t@CNXo
            MdbInfo := TMdbFile(MdbFiles[i]);
            DoMessage(Format('[%2d/%2d] ͊Jn %s', [i + 1, Count, MdbInfo.DispName]));

            //  ڑmFς݂̏ꍇ̂
            if (MdbInfo.Status = fsNoProblem) then
            begin
                ForceDirectories(OutPath);
                DoAnalyzeFile(OutPath, MdbInfo.FileName, MdbInfo.Password);
            end
            else
            begin
                DoMessage('ڑ`FbN̂߃X[܂B');
            end;
        end;

        DoMessage(StringOfChar('-', 50));
        Tick := GetTickCount() - Tick;
        DoMessage(Format('v %.3fb', [Tick / 1000]));
    finally
        PnlButtons.Enabled := true;
        Screen.Cursor := crDefault;
    end;

    //  o̓tH_J
    if DirectoryExists(OutPath) then
    begin
        if (SimpleMessage('o̓tH_J܂H', mtConfirmation, [mbYes, mbNo]) = mrYes) then
        begin
            SimpleExecute(OutPath);
        end;
    end;
end;

//  ---------------------------------------------------------------------------
//  ͖{
procedure TMainForm.DoAnalyzeFile(const OutPath, FileName, Password: string);
var
    AdoJet: TAdoxJetBase;
    OutFile: string;
    F: TextFile;
    i, j, k, TableCount, ViewCount: integer;
    CNN, CAT, TABLE, COLUMN, VIEW: OleVariant;
    PROP: OleVariant;
label
    HELL_HELL_HELL;
begin
    //  o͐挈
    OutFile := OutPath + ExtractFileName(ChangeFileExt(FileName, '.html'));

    AdoJet := TAdoxJetBase.Create;
    try
        try
            //  ڑ
            AdoJet.OpenConnection(FileName, Password);
            DoMessage('ڑ܂B', 1);

            //  o͐͂
            AssignFile(F, OutFile);
            Rewrite(F);

            //  IuWFNgoĂ
            CNN := AdoJet.CNN;
            CAT := AdoJet.CAT;

            //  =====================================================
            //  HTML̃wb_
            Writeln(F, '<!DOCTYPE html PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN">');
            Writeln(F, '<html lang="ja"><head>');
            Writeln(F, '<meta http-equiv="Content-Type" content="text/html; charset=Shift_JIS">');
            Writeln(F, '<meta http-equiv="Content-Script-Type" content="text/javascript">');
            Writeln(F, '<meta http-equiv="Content-Style-Type" content="text/css">');
            Writeln(F, '<style type="text/css"><!--');
            Writeln(F, 'body,table{font-size:x-small;}body{font-family:sans-serif;}table{font-family:monospace;}');
            Writeln(F, '--></style>');
            //  ^Cg
            Writeln(F, Format('<title>%s ͌</title>', [ExtractFileName(FileName)]));
            Writeln(F, '</head><body>');

            //  HTML̃{fB
            Writeln(F, Format('<h1>%s ͌</h1>', [ExtractFileName(FileName)]));
            Writeln(F, Format('<p>Generated by %s at %s', [ExtractFileName(ParamStr(0)), FormatDateTime('yyyy/mm/dd hh:nn:ss', Now())]));

            //  e[u邩ǂ`FbN
            TableCount := CAT.Tables.Count;
            //  ΂̏XLbv
            if (TableCount = 0) then goto HELL_HELL_HELL;

            //  =====================================================
            //   1 
            DoMessage('e[uꗗo͒...', 1);
            Writeln(F, '<h2>e[uꗗ</h2>');
            Writeln(F, '<p><table border=1 cellpadding=3 cellspacing=0>');
            //  <TABLE>̌Œ^Cg쐬
            Writeln(F, '<tr><th nowrap>NO</th><th nowrap>e[u</th><th nowrap></th><th nowrap>쐬</th><th nowrap>XV</th>');
            //  <TABLE>̕^Cg쐬̂߂Ƀe[uIuWFNgo
            TABLE := CAT.Tables.Item[0];
            //  vpeB񋓂
            for i := 0 to TABLE.Properties.Count - 1 do
            begin
                PROP := TABLE.Properties.Item[i];
                Writeln(F, Format('<th>%s</th>', [PROP.Name]));
            end;
            Writeln(F, '</tr>');
            //  e[ũvpeBo
            for i := 0 to TableCount - 1 do
            begin
                //  e[uIuWFNgo
                TABLE := CAT.Tables.Item[i];
                Writeln(F, '<tr>');
                //  ŒvpeBo
                Writeln(F, Format('<td nowrap>%d</td>', [i + 1]));
                Writeln(F, Format('<td nowrap>%s</td>', [TABLE.Name]));
                Writeln(F, Format('<td nowrap>%s</td>', [TABLE.type]));
                Writeln(F, Format('<td nowrap>%s</td>', [VarToStr(TABLE.DateCreated)]));
                Writeln(F, Format('<td nowrap>%s</td>', [VarToStr(TABLE.DateModified)]));
                //  vpeBo
                for j := 0 to TABLE.Properties.Count - 1 do
                begin
                    PROP := TABLE.Properties.Item[j];
                    Writeln(F, Format('<td>%s@</td>', [PROP.Value]));
                end;
                Writeln(F, '</tr>');
            end;
            Writeln(F, '</table>');

            //  =====================================================
            //   2 
            DoMessage('L[擾...', 1);
{
//  Key Object 

//-----------------------------------------------------------------------------
//  KeyIuWFNgpvpeB
procedure TAdoxJetTest.EnumKeyProperty(key: OleVariant);
var
    s: string;
begin
    s := Format('L[   %s', [key.Name]);
    FStrings.Add(s);
    s := Format('L[   %s', [S_KeyTypeEnum[Integer(key.type)]]);
    FStrings.Add(s);
    s := Format('֘A\     %s', [key.RelatedTable]);
    FStrings.Add(s);
    s := Format('폜[ %s', [S_RuleEnum[Integer(key.DeleteRule)]]);
    FStrings.Add(s);
    s := Format('XV[ %s', [S_RuleEnum[Integer(key.UpdateRule)]]);
    FStrings.Add(s);
end;

//-----------------------------------------------------------------------------
//  KeyIuWFNgpColumnIuWFNgpvpeB
procedure TAdoxJetTest.EnumKeyColumnProperty(column: OleVariant);
var
    s: string;
begin
    s := Format('񖼏     %s', [column.Name]);
    FStrings.Add(s);
    s := Format('֘A񖼏 %s', [column.RelatedColumn]);
    FStrings.Add(s);
end;

//-----------------------------------------------------------------------------
//  KeyIuWFNgpColumnIuWFNg
procedure TAdoxJetTest.EnumKeyColumns(key: OleVariant);
var
    i: integer;
    CLM: OleVariant;
begin
    if FOpened then
    begin
        for i := 0 to key.Columns.Count - 1 do
        begin
            try
                CLM := key.Columns.Item[i];
                FStrings.Add(StringOfChar('*', 20));
                EnumKeyColumnProperty(CLM);
            except
                on E:Exception do FStrings.Add(E.Message);
            end;
        end;
    end;
end;

//-----------------------------------------------------------------------------
//  KeyIuWFNg
procedure TAdoxJetTest.EnumKeys(table: OleVariant);
var
    i: integer;
    KEY: OleVariant;
begin
    if FOpened then
    begin
        for i := 0 to table.Keys.Count - 1 do
        begin
            KEY := table.Keys.Item[i];
            FStrings.Add(StringOfChar('-', 20));
            EnumKeyColumns(KEY);
            EnumKeyProperty(KEY);
        end;
    end;
end;

//  Index Object 

//-----------------------------------------------------------------------------
//  IndexIuWFNgpvpeB
procedure TAdoxJetTest.EnumIndexProperty(idx: OleVariant);
var
    s: string;
begin
    s := Format('CfbNX %s', [idx.Name]);
    FStrings.Add(s);
    if (idx.Unique) then s := 'must be unique' else s := 'must not be unique';
    FStrings.Add(s);
    if (idx.PrimaryKey) then s := 'is the primary key' else s := 'is not the primary key';
    FStrings.Add(s);
    s := Format('IndexNulls   %s', [S_AllowNullsEnum[Integer(idx.IndexNulls)]]);
    FStrings.Add(s);
    if (idx.Clustered) then s := 'is clustered' else s := 'is not clustered';
    FStrings.Add(s);
end;

//-----------------------------------------------------------------------------
//  IndexIuWFNgpColumnIuWFNgpvpeB
procedure TAdoxJetTest.EnumIndexColumnProperty(column: OleVariant);
var
    s: string;
begin
    s := Format('column name   %s', [column.Name]);
    FStrings.Add(s);
//    s := Format('SortOrder     %s', [SortOrderEnumStr[TSortOrderEnum(column.SortOrder)]]);
//    FStrings.Add(s);
end;

//-----------------------------------------------------------------------------
//  IndexIuWFNgpColumnIuWFNg
procedure TAdoxJetTest.EnumIndexColumns(idx: OleVariant);
var
    i: integer;
    CLM: OleVariant;
begin
    if FOpened then
    begin
        for i := 0 to idx.Columns.Count - 1 do
        begin
            try
                CLM := idx.Columns.Item[i];
                FStrings.Add(StringOfChar('*', 20));
                EnumIndexColumnProperty(CLM);
            except
                on E:Exception do FStrings.Add(E.Message);
            end;
        end;
    end;
end;

//-----------------------------------------------------------------------------
//  IndexIuWFNg
procedure TAdoxJetTest.EnumIndexs(table: OleVariant);
var
    i: integer;
    IDX: OleVariant;
begin
    if FOpened then
    begin
        for i := 0 to table.Indexes.Count - 1 do
        begin
            IDX := table.Indexes.Item[i];
            FStrings.Add(StringOfChar('+', 20));
            EnumIndexColumnProperty(IDX);
            EnumIndexColumns(IDX);
        end;
    end;
end;
}
            //  =====================================================
            //   3 
            DoMessage('CfbNX擾...', 1);

            //  =====================================================
            //   4 
            DoMessage('tB[h\o͒...', 1);
            Writeln(F, '<h2>tB[h\</h2>');

            //  ee[uƂɏo
            for i := 0 to TableCount - 1 do
            begin
                //  e[uIuWFNgo
                TABLE := CAT.Tables.Item[i];
                //  ^Cgo
                DoMessage(Format('[%2d/%2d] %s', [i + 1, TableCount, TABLE.Name]), 2);
                Writeln(F, Format('<h3>%s</h3>', [TABLE.Name]));
                //  tB[h`FbN
                if (TABLE.Columns.Count = 0) then continue;
                Writeln(F, '<p><table border=1 cellpadding=3 cellspacing=0>');
                //  <TABLE>̌Œ^Cg쐬
                Writeln(F, '<tr><th nowrap>NO</th><th nowrap>񖼏</th><th nowrap>^</th><th nowrap></th><th nowrap>oCg</th><th nowrap>l^</th><th nowrap>x</th>');
                //  <TABLE>̕^Cg쐬̂߂ɃtB[hIuWFNgo
                COLUMN := TABLE.Columns.Item[0];
                //  vpeB񋓂
                for j := 0 to COLUMN.Properties.Count - 1 do
                begin
                    PROP := COLUMN.Properties.Item[j];
                    Writeln(F, Format('<th>%s</th>', [PROP.Name]));
                end;
                Writeln(F, '</tr>');
                //  tB[h̃vpeBo
                for j := 0 to TABLE.Columns.Count - 1 do
                begin
                    //  tB[hIuWFNgo
                    COLUMN := TABLE.Columns.Item[j];
                    Writeln(F, '<tr>');
                    //  ŒvpeBo
                    Writeln(F, Format('<td nowrap>%d</td>', [j + 1]));
                    Writeln(F, Format('<td nowrap>%s</td>', [COLUMN.Name]));
                    Writeln(F, Format('<td nowrap>%s</td>', [DataTypeEnum2Str(COLUMN.Type)]));
                    Writeln(F, Format('<td nowrap>%s</td>', [ColumnAttributesEnum2Str(COLUMN.Attributes)]));
                    Writeln(F, Format('<td nowrap>%s</td>', [COLUMN.DefinedSize]));
                    Writeln(F, Format('<td nowrap>%s</td>', [COLUMN.NumericScale]));
                    Writeln(F, Format('<td nowrap>%s</td>', [COLUMN.Precision]));
                    //  vpeBo
                    for k := 0 to COLUMN.Properties.Count - 1 do
                    begin
                        PROP := COLUMN.Properties.Item[k];
                        Writeln(F, Format('<td>%s@</td>', [PROP.Value]));
                    end;
                    Writeln(F, '</tr>');
                end;
                Writeln(F, '</table>');
            end;

            //  =====================================================
            //   5 
            //  r[邩ǂ`FbN
            ViewCount := CAT.Views.Count;
            //  ΂̏XLbv
            if (ViewCount = 0) then goto HELL_HELL_HELL;

            DoMessage('r[ꗗo͒...', 1);
            Writeln(F, '<h2>r[ꗗ</h2>');
            Writeln(F, '<p><table border=1 cellpadding=3 cellspacing=0>');
            //  <TABLE>̌Œ^Cg쐬
            Writeln(F, '<tr><th nowrap>NO</th><th nowrap>r[</th><th nowrap>쐬</th><th nowrap>XV</th>');
            Writeln(F, '</tr>');
            //  r[̃vpeBo
            for i := 0 to ViewCount - 1 do
            begin
                //  r[IuWFNgo
                VIEW := CAT.Views.Item[i];
                Writeln(F, '<tr>');
                //  ŒvpeBo
                Writeln(F, Format('<td nowrap>%d</td>', [i + 1]));
                Writeln(F, Format('<td nowrap>%s</td>', [VIEW.Name]));
                Writeln(F, Format('<td nowrap>%s</td>', [VarToStr(VIEW.DateCreated)]));
                Writeln(F, Format('<td nowrap>%s</td>', [VarToStr(VIEW.DateModified)]));
                Writeln(F, '</tr>');
            end;
            Writeln(F, '</table>');

            //  =====================================================
            //   6 
            DoMessage('r[R}ho͒...', 1);

//  GOTO HELL
HELL_HELL_HELL:

            //  =====================================================
            //   7 
            DoMessage('ڑ̃vpeBo͒...', 1);
            Writeln(F, '<h2>ڑ̃vpeB</h2>');
            Writeln(F, '<p><table border=1 cellpadding=3 cellspacing=0>');
            Writeln(F, '<tr><th nowrap>NO</th><th nowrap>vpeB</th><th nowrap>ݒl</th></tr>');
            for i := 0 to CNN.Properties.Count - 1 do
            begin
                PROP := CNN.Properties.Item[i];
                Writeln(F, Format('<tr><td nowrap>%d</td><td nowrap>%s</td><td nowrap>%s@</td></tr>', [i + 1, PROP.Name, PROP.Value]));
            end;
            Writeln(F, '</table>');

            //  =====================================================
            //  HTML̃e[
            Writeln(F, '</body></html>');

            //  t@C
            CloseFile(F);

            //  ؒf
            AdoJet.CloseConnection();
            DoMessage('ؒf܂B', 1);
        except
            //  蔭
            on E:Exception do DoMessage(E.Message);
        end;
    finally
        AdoJet.Free;
    end;
end;



end.



