unit iadoconst;

interface

uses
    SysUtils;

const
    //  DataTypeEnum
    adEmpty            =     0; //  Specifies no value (DBTYPE_EMPTY).
    adSmallInt         =     2; //  Indicates a two-byte signed integer (DBTYPE_I2).
    adInteger          =     3; //  Indicates a four-byte signed integer (DBTYPE_I4).
    adSingle           =     4; //  Indicates a single-precision floating-point value (DBTYPE_R4).
    adDouble           =     5; //  Indicates a double-precision floating-point value (DBTYPE_R8).
    adCurrency         =     6; //  Indicates a currency value (DBTYPE_CY). Currency is a fixed-point number with four digits to the right of the decimal point. It is stored in an eight-byte signed integer scaled by 10,000.
    adDate             =     7; //  Indicates a date value (DBTYPE_DATE). A date is stored as a double, the whole part of which is the number of days since December 30, 1899, and the fractional part of which is the fraction of a day.
    adBSTR             =     8; //  Indicates a null-terminated character string (Unicode) (DBTYPE_BSTR).
    adIDispatch        =     9; //  Indicates a pointer to an IDispatch interface on a COM object (DBTYPE_IDISPATCH).
                                //  This data type is currently not supported by ADO. Usage may cause unpredictable results.
    adError            =    10; //  Indicates a 32-bit error code (DBTYPE_ERROR).
    adBoolean          =    11; //  Indicates a boolean value (DBTYPE_BOOL).
    adVariant          =    12; //  Indicates an Automation Variant (DBTYPE_VARIANT).
                                //  This data type is currently not supported by ADO. Usage may cause unpredictable results.
    adIUnknown         =    13; //  Indicates a pointer to an IUnknown interface on a COM object (DBTYPE_IUNKNOWN).
                                //  This data type is currently not supported by ADO. Usage may cause unpredictable results.
    adDecimal          =    14; //  Indicates an exact numeric value with a fixed precision and scale (DBTYPE_DECIMAL).
    adTinyInt          =    16; //  Indicates a one-byte signed integer (DBTYPE_I1).
    adUnsignedTinyInt  =    17; //  Indicates a one-byte unsigned integer (DBTYPE_UI1).
    adUnsignedSmallInt =    18; //  Indicates a two-byte unsigned integer (DBTYPE_UI2).
    adUnsignedInt      =    19; //  Indicates a four-byte unsigned integer (DBTYPE_UI4).
    adBigInt           =    20; //  Indicates an eight-byte signed integer (DBTYPE_I8).
    adUnsignedBigInt   =    21; //  Indicates an eight-byte unsigned integer (DBTYPE_UI8).
    adFileTime         =    64; //  Indicates a 64-bit value representing the number of 100-nanosecond intervals since January 1, 1601 (DBTYPE_FILETIME).
    adGUID             =    72; //  Indicates a globally unique identifier (GUID) (DBTYPE_GUID).
    adBinary           =   128; //  Indicates a binary value (DBTYPE_BYTES).
    adChar             =   129; //  Indicates a string value (DBTYPE_STR).
    adWChar            =   130; //  Indicates a null-terminated Unicode character string (DBTYPE_WSTR).
    adNumeric          =   131; //  Indicates an exact numeric value with a fixed precision and scale (DBTYPE_NUMERIC).
    adUserDefined      =   132; //  Indicates a user-defined variable (DBTYPE_UDT).
    adDBDate           =   133; //  Indicates a date value (yyyymmdd) (DBTYPE_DBDATE).
    adDBTime           =   134; //  Indicates a time value (hhmmss) (DBTYPE_DBTIME).
    adDBTimeStamp      =   135; //  Indicates a date/time stamp (yyyymmddhhmmss plus a fraction in billionths) (DBTYPE_DBTIMESTAMP).
    adChapter          =   136; //  Indicates a four-byte chapter value that identifies rows in a child rowset (DBTYPE_HCHAPTER).
    adPropVariant      =   138; //  Indicates an Automation PROPVARIANT (DBTYPE_PROP_VARIANT).
    adVarNumeric       =   139; //  Indicates a numeric value (Parameter object only).
    adVarChar          =   200; //  Indicates a string value.
    adLongVarChar      =   201; //  Indicates a long string value.
    adVarWChar         =   202; //  Indicates a null-terminated Unicode character string.
    adLongVarWChar     =   203; //  Indicates a long null-terminated Unicode string value.
    adVarBinary        =   204; //  Indicates a binary value (Parameter object only).
    adLongVarBinary    =   205; //  Indicates a long binary value.
    AdArray            = $2000; //  A flag value, always combined with another data type constant, that indicates an array of that other data type.

    //  KeyTypeEnum
    adKeyPrimary    = 1;
    adKeyForeign    = 2;
    adKeyUnique     = 3;
    S_KeyTypeEnum: array[adKeyPrimary..adKeyUnique] of string
                    = ('PK', 'FK', 'Unique');

    //  ColumnAttributesEnum
    adColFixed      = 1;
    adColNullable   = 2;
    S_ColumnAttributesEnum: array[adColFixed..adColNullable] of string
                    = ('Fixed Length', 'Nullable');
    //  RuleEnum
    adRINone        = 0;
    adRICascade     = 1;
    adRISetNull     = 2;
    adRISetDefault  = 3;
    S_RuleEnum: array[adRINone..adRISetDefault] of string
                    = ('None', 'Cascade', 'SetNull', 'SetDefault');

    //  AllowNullsEnum
    adIndexNullsAllow       = 0;
    adIndexNullsDisallow    = 1;
    adIndexNullsIgnore      = 2;
    adIndexNullsIgnoreAny   = 4;
    S_AllowNullsEnum: array[adIndexNullsAllow..adIndexNullsIgnoreAny] of string
                    = ('Allow', 'Disallow', 'Ignore', '', 'IgnoreAny');

    //  SortOrderEnum
    adSortAscending     = 1;
    adSortDescending    = 2;
    S_SortOrderEnum: array[adSortAscending..adSortDescending] of string
                    = ('Ascending', 'Descending');

function DataTypeEnum2Str(const AdoType: integer): string;
function ColumnAttributesEnum2Str(const AdoType: integer): string;

implementation

//-----------------------------------------------------------------------------
//  tO܂ł邩?
//function FlagInclude(const Flags, Flag: integer): Boolean;
//begin
//    Result := ((Flags and Flag) = Flag);
//end;

//-----------------------------------------------------------------------------
//  f[^^
function DataTypeEnum2Str(const AdoType: integer): string;
var
    aType: integer;
    Temp: string;
begin
    //  ztO܂ނǂ
    aType := AdoType;
    if (aType >= AdArray) then
    begin
        Dec(aType, AdArray);
        Temp := '[Array]';
    end
    else
    begin
        Temp := '';
    end;
    //  ^
    if (aType = adEmpty           ) then Result := 'Empty'            else
    if (aType = adSmallInt        ) then Result := 'SmallInt'         else
    if (aType = adInteger         ) then Result := 'Integer'          else
    if (aType = adSingle          ) then Result := 'Single'           else
    if (aType = adDouble          ) then Result := 'Double'           else
    if (aType = adCurrency        ) then Result := 'Currency'         else
    if (aType = adDate            ) then Result := 'Date'             else
    if (aType = adBSTR            ) then Result := 'BSTR'             else
    if (aType = adIDispatch       ) then Result := 'IDispatch'        else
    if (aType = adError           ) then Result := 'Error'            else
    if (aType = adBoolean         ) then Result := 'Boolean'          else
    if (aType = adVariant         ) then Result := 'Variant'          else
    if (aType = adIUnknown        ) then Result := 'IUnknown'         else
    if (aType = adDecimal         ) then Result := 'Decimal'          else
    if (aType = adTinyInt         ) then Result := 'TinyInt'          else
    if (aType = adUnsignedTinyInt ) then Result := 'UnsignedTinyInt'  else
    if (aType = adUnsignedSmallInt) then Result := 'UnsignedSmallInt' else
    if (aType = adUnsignedInt     ) then Result := 'UnsignedInt'      else
    if (aType = adBigInt          ) then Result := 'BigInt'           else
    if (aType = adUnsignedBigInt  ) then Result := 'UnsignedBigInt'   else
    if (aType = adFileTime        ) then Result := 'FileTime'         else
    if (aType = adGUID            ) then Result := 'GUID'             else
    if (aType = adBinary          ) then Result := 'Binary'           else
    if (aType = adChar            ) then Result := 'Char'             else
    if (aType = adWChar           ) then Result := 'WChar'            else
    if (aType = adNumeric         ) then Result := 'Numeric'          else
    if (aType = adUserDefined     ) then Result := 'UserDefined'      else
    if (aType = adDBDate          ) then Result := 'DBDate'           else
    if (aType = adDBTime          ) then Result := 'DBTime'           else
    if (aType = adDBTimeStamp     ) then Result := 'DBTimeStamp'      else
    if (aType = adChapter         ) then Result := 'Chapter'          else
    if (aType = adPropVariant     ) then Result := 'PropVariant'      else
    if (aType = adVarNumeric      ) then Result := 'VarNumeric'       else
    if (aType = adVarChar         ) then Result := 'VarChar'          else
    if (aType = adLongVarChar     ) then Result := 'LongVarChar'      else
    if (aType = adVarWChar        ) then Result := 'VarWChar'         else
    if (aType = adLongVarWChar    ) then Result := 'LongVarWChar'     else
    if (aType = adVarBinary       ) then Result := 'VarBinary'        else
    if (aType = adLongVarBinary   ) then Result := 'LongVarBinary'    else
    if (aType = adArray           ) then Result := 'Array'            else
    Result := '';

    Result := Result + Temp;
end;

//-----------------------------------------------------------------------------
//  tB[h̑
function ColumnAttributesEnum2Str(const AdoType: integer): string;
begin
    //  Œ蒷ǂ
    if (AdoType = adColFixed) then
        Result := S_ColumnAttributesEnum[adColFixed]
    //  NULLlǂ
    else
    if (AdoType = adColNullable) then
        Result := S_ColumnAttributesEnum[adColNullable]
    //  
    else
    if (AdoType = adColFixed + adColNullable) then
        Result := S_ColumnAttributesEnum[adColFixed] + ',' + S_ColumnAttributesEnum[adColNullable]
    //  ǂłȂ
    else
        Result := '';
end;

end.

